#include "stdafx.h"
#include "CalcProb.h"

using namespace System;
using namespace myLib;

namespace myLib {

	//
	//     {uvwwԂ߂̐w[]vA|
	//				8.3߁uϕ̐lvZv
	//
	double AdaptiveGL::integral(double a, double b, double (*func)(double)){
		f = func;
		double v = 0.0;
		calcIntegral( v, a, b );
		return v;
	}


	void AdaptiveGL::calcIntegral(double & s0, double a, double b){
		double s1 = Gauss_Legendre( a, 0.5*(a + b) );
		double s2 = Gauss_Legendre( 0.5*(a + b), b );
		if (((Math::Abs( s0 - s1 - s2 ) < (1.0e-14)*Math::Abs(s1 + s2))
			|| 
			((Math::Abs(s0) + Math::Abs(s1 + s2)) < 1.0e-14))
			&&
			(Math::Abs(b - a) < 1.0)){
				s0 = s1 + s2;
				return;
		}
		else{
				calcIntegral( s1, a, 0.5*(a + b) );
				calcIntegral( s2, 0.5*(a + b), b );
				s0 = s1 + s2;
				return;
		}
	}


	double AdaptiveGL::Gauss_Legendre(double a, double b){
		double x0[9] = {
		    			   -9.68160239507626090E-0001,
						   -8.36031107326635794E-0001,
						   -6.13371432700590397E-0001,
						   -3.24253423403808929E-0001,
							0.00000000000000000E+0000,
							3.24253423403808929E-0001,
							6.13371432700590397E-0001,
							8.36031107326635794E-0001,
							9.68160239507626090E-0001
		};
		double w[9] = {
							8.12743883615744120E-0002,
							1.80648160694857404E-0001,
							2.60610696402935462E-0001,
							3.12347077040002840E-0001,
							3.30239355001259763E-0001,
							3.12347077040002840E-0001,
							2.60610696402935462E-0001,
							1.80648160694857404E-0001,
							8.12743883615744120E-0002  
		};

		double v1 = 0.0;
		double i_w = b - a;
		double xi;
		for (int i = 0; i < 9; i++){
			xi = (0.5 * i_w * x0[i]) + (0.5 * (b + a));
			v1 += w[i] * f(xi);
		}
		v1 = 0.5 * i_w * v1;
		return v1;
	}


	double my_power( double a, double b ){
		return Math::Exp(b * Math::Log(a));
	}


	namespace chi_df {
			double df;
	}


	double Chi_kernel( double x ){
		using namespace chi_df;
		return my_power( x, df/2.0 - 1.0 ) * Math::Exp(-x/2.0);
	}


	double gamma( double r ){
		if (r > 1.25){
						return (r - 1) * gamma(r - 1);
		}else{
			if (r > 0.75){
						return 1.0;
			}else{
				return Math::Sqrt(Math::PI);
			}
		}
	}


	double sqr( double x ){
			return x * x;
	}

	double Normal_Dist( double x ){
		return Math::Exp(-0.5 * sqr(x));
	}


	double Cum_Chi_distribution( double chi ){
		using namespace chi_df;
		if (chi <= 0.0){
					return 0.0;
		}else if(df <= 1.0){
					AdaptiveGL adGL;
					return adGL.integral( -Math::Sqrt(chi), Math::Sqrt(chi), Normal_Dist) / Math::Sqrt(2.0 * Math::PI);		
		}else{
					AdaptiveGL adGL;
					double v = adGL.integral( 0.0, chi, Chi_kernel );
					return v / (gamma(df/2.0) * my_power( 2.0, df/2.0 ));
		}
	}

	double CalcChi2Prob( double chi2, double dfv ){
			using namespace chi_df;
			df = dfv;
			double p = Cum_Chi_distribution( chi2 );
			if (p < 0.0){
						p = 0.0;
			}else if (p > 1.0){
						p = 1.0;
			};

			return 1.0 - p;
	}



}  //  myLib

